import type { ServerLoadEvent } from '@sveltejs/kit';
import { type ILocalizationService, type LoadFunction, type LoadResult, type ServiceConfig } from './service.svelte.js';
type ImportLoadResult = {
    default: LoadResult;
};
export type ImportLoads = Record<string, () => Promise<ImportLoadResult>>;
type CommonServiceConfig = Omit<ServiceConfig, 'activeLocale'>;
type InitialServiceConfig = Partial<ServiceConfig> & Pick<ServiceConfig, 'activeLocale'>;
type ExtractedLocales = {
    availableLocales: string[];
    requestedLocales: string[];
    activeLocale: string;
};
type ActiveLocaleSearchOptions = {
    params?: string[];
    searchParams?: string[];
    cookies?: string[];
};
type FactoryConfig = {
    readonly browser: boolean;
    readonly contextName: string;
    readonly importDirPath: string;
    readonly importLoads: ImportLoads;
};
type ImportLoadFactory = (importFilePath: string) => LoadFunction;
export interface ILocalizationFactory {
    configure(config: FactoryConfig): void;
    config: FactoryConfig;
    setCommonServiceConfig(config: CommonServiceConfig): void;
    commonServiceConfig: CommonServiceConfig;
    importLoaderFactory(): ImportLoadFactory;
    extractLocales(event: ServerLoadEvent, searchOptions?: ActiveLocaleSearchOptions): ExtractedLocales;
    setContextService(service: ILocalizationService): void;
    getContextService(): ILocalizationService;
    initialLoadLocalizations(pathname: string, config: InitialServiceConfig): Promise<ILocalizationService>;
}
export declare const LocalizationFactory: ILocalizationFactory;
export {};
